﻿//******************************************************** 
// Orbitální elementy 
//********************************************************
function OrbitalElements()
{
 this.A = 0.0;    
 this.EC = 0.0;     
 this.IN = 0.0;   
 this.OM = 0.0;
 this.W = 0.0;    
 this.MA = 0.0;      
 this.E = 0.0;    
 this.v = 0.0;
 this.N = 0.0;
 this.OPCStoHAE = Matrix.Identity();
 this.HAEtoOPCS = Matrix.Identity();
 this.SetMatrixes();
}
//========================================================
OrbitalElements.FromThis = function()
{
 var OE = new OrbitalElements();
 
 OE.A = this.A;    
 OE.EC = this.EC;     
 OE.IN = this.IN;   
 OE.OM = this.OM;
 OE.W = this.W;    
 OE.MA = this.MA;      
 OE.E = this.E;    
 OE.v = this.v;
 OE.N = this.N;
 OE.SetMatrixes();
 
 return OE;
}
//========================================================
OrbitalElements.KeplerEquation = function(EC, MA)
{
 var E0 = 0.0;
 var En = Angle.ToRad(MA);
 var i = 0;

 MA = En;
 do
 {
  i++;
  E0 = En;
  En = MA + EC * Math.sin(E0);
 }
 while((Math.abs(En - E0) > 0.000001) && i >= 1000);
 
 return Angle.ToDeg(En);
}
//========================================================
OrbitalElements.TrueAnomaly = function(EC, E)
{
 var sinv = 0;
 var cosv = 0;
 
 E = Angle.ToRad(E);
 sinv = Math.sqrt(1 - Math.pow(EC, 2)) * Math.sin(E) / (1 - EC * Math.cos(E));
 cosv = (Math.cos(E) - EC) / (1 - EC * Math.cos(E));
 
 return Angle.GetAngle(sinv, cosv);
}
//========================================================
OrbitalElements.prototype.CalcAnomalies = function()
{
 this.E = OrbitalElements.KeplerEquation(this.EC, this.MA);
 this.v = OrbitalElements.TrueAnomaly(this.EC, this.E);
}
//========================================================
OrbitalElements.prototype.ExcentricAnomaly = function(EC, v)
{
 var E = 0;
 v = Angle.ToRad(v);

 if(EC < 1)
 {
  var sin = Math.sqrt(1 - EC * EC) * Math.sin(v) / (EC * Math.cos(v) + 1);
  var cos = (EC + Math.cos(v)) / (EC * Math.cos(v) + 1);
  E = Angle.GetAngle(sin, cos);
 }
 
 return E;
}
//============================================================
OrbitalElements.prototype.MeanAnomaly = function(EC, E)
{
 var MA;
 
 E = Angle.ToRad(E);  
 MA= E - EC * Math.sin(E);
 return Angle.ToDeg(MA);
}
//============================================================
OrbitalElements.prototype.ToStateVector = function()
{
 var SV = new StateVector();
 
 if(this.EC < 1)
  SV.X = Vector.FromSpherical(this.A * (1 - this.EC * Math.cos(Angle.ToRad(this.E))), this.v, 0.0);
  
 if(this.EC > 1)
  SV.X = Vector.FromSpherical(this.A * (1 - this.EC * Math.cosh(Angle.ToRad(this.E))), this.v, 0.0); 
  
 SV.V.X[0] = -Math.sqrt(Konst.GMSUN / (this.A * (1 - this.EC * this.EC))) * Math.sin(Angle.ToRad(this.v));
 SV.V.X[1] =  Math.sqrt(Konst.GMSUN / (this.A * (1 - this.EC * this.EC))) * (this.EC + Math.cos(Angle.ToRad(this.v)));
 SV.V.X[2] =  0;
 SV.V.ToSpherical();
 
 return SV;
}
//========================================================
// SV je v ICRF
OrbitalElements.prototype.FromStateVector = function(SV)
{
 var sv = StateVector.MMULT(Matrix.Identity(), SV); 
 sv.X = Vector.MULT(Konst.AU, sv.X);
 var b = Vector.PROD(sv.X, sv.V);
 var e;
 var retro = 1;
 var OM = b.Fi + 90;
 var IN = 90 - b.Theta;
 var W, A, v, EC, E, MA, N;
 
 if(IN < 0.0000000000001) OM = 0;
 if(Math.abs(IN - 180) <= 0.0000000000001)
 {
  retro = -1;
  OM = 180;
 }
 
 A = 1.0 / (2 / sv.X.R - sv.V.R * sv.V.R / Konst.GMSUN);
 e = Vector.PROD(sv.V, b); 
 e = Vector.DIFF(Vector.MULT(1.0 / Konst.GMSUN, e), Vector.Unitary(sv.X));
 e = Matrix.VMULT(Matrix.MULT(Matrix.RotX(IN), Matrix.RotZ(OM)), e);
 W = e.Fi * retro;
 W = Angle.ToAngle360(W);
 e = Matrix.VMULT(Matrix.MULT(Matrix.RotX(IN), Matrix.RotZ(OM)), sv.X);
 v = retro * e.Fi - W;
 EC = Math.sqrt(1 - b.R * b.R / (Konst.GMSUN * A));
 E = this.ExcentricAnomaly(EC, v);
 MA = this.MeanAnomaly(EC, E);
 N = Angle.ToDeg(retro * Math.sqrt(Konst.GMSUN / Math.pow(Math.abs(A), 3)) * 86400);
 
 this.OM = OM;
 this.IN = IN;
 this.W = W;
 this.A = A / Konst.AU;
 this.EC = EC;
 this.v = v;
 this.E = E;
 this.MA = MA;
 this.SetMatrixes();
}
//========================================================
OrbitalElements.prototype.SetMatrixes = function()
{
 this.OPCStoHAE = Matrix.MULT(Matrix.RotZ(-this.OM), Matrix.MULT(Matrix.RotX(-this.IN), Matrix.RotZ(-this.W)));
 this.HAEtoOPCS = Matrix.MULT(Matrix.RotZ(this.W), Matrix.MULT(Matrix.RotX(this.IN), Matrix.RotZ(this.OM)));
}
//******************************************************** 

//******************************************************** 
// Efemeridy 
//********************************************************
function Body(A0, A1, EC0, EC1, IN0, IN1, OM0, OM1, PI0, PI1, L0, L1, NPRA, NPDE)
{
 this.A0 = A0;    this.A1 = A1;      this.EC0 = EC0;  this.EC1 = EC1;
 this.IN0 = IN0;  this.IN1 = IN1;    this.OM0 = OM0;  this.OM1 = OM1;
 this.PI0 = PI0;  this.PI1 = PI1;    this.L0 = L0;    this.L1 = L1;
 this.W0 = 0;
 this.N = L1 - PI1;
  
 this.OE = new OrbitalElements();
 this.SV = new StateVector();
 this.CC = new Array();
 
 this.NPRA = NPRA;   
 this.NPDE = NPDE;
}
//========================================================
Body.prototype.CalcPosition = function(delta2000, useDE405, calcVelocity)
{
 this.CalcActualOE(delta2000);
 this.OE.CalcAnomalies();
 
 if(useDE405 && this.CalcDE405(delta2000 + Konst.JD2000, calcVelocity))
  return;
 
 this.SV = this.OE.ToStateVector();
 this.OPCStoICRF();
}
//========================================================
Body.prototype.CalcActualOE = function(delta)
{
 this.OE.A   = this.A0  + this.A1  * delta;
 this.OE.EC  = this.EC0 + this.EC1 * delta;
 this.OE.IN  = this.IN0 + this.IN1 * delta;
 this.OE.OM  = this.OM0 + this.OM1 * delta;
 this.OE.W   = this.PI0 + this.PI1 * delta - this.OE.OM;
 this.W0  = this.PI0 - this.OE.OM;
 this.OE.MA   = this.L0  + this.L1  * delta - this.OE.OM - this.OE.W;
}
//========================================================
Body.prototype.CalcDE405 = function(jd, calcVelocity)
{
 var isCalc = false;
 var startIdx = 0;
 var SubIntervals = 0;
 var Interval = 0;

 for(var i = 0; i < this.CC.length; i++)
 {
  if(!(jd >= this.CC[i][2] && jd <= this.CC[i][3]) && isCalc)
   break;
 
  if(jd >= this.CC[i][2] && jd <= this.CC[i][3] && isCalc && Interval == this.CC[i][0])
   SubIntervals++;
 
  if(jd >= this.CC[i][2] && jd <= this.CC[i][3] && !isCalc)
  {
   isCalc = true;
   startIdx = i;
   SubIntervals = 1;
   Interval = this.CC[i][0];
  }
 }
 
 if(isCalc)
 {
  var NSEG;
  var TSEG;
  var D;
  var TINT;
  var IEPH;
  var NumCoef;
  var X, Y, Z;
  var VX, VY, VZ;
  
  
  D = this.CC[startIdx][3] - this.CC[startIdx][2];
  TINT = (jd - this.CC[startIdx][2]) / D;
  IEPH = parseInt(Math.floor(TINT));
  TINT = (TINT - IEPH) * SubIntervals;
  NSEG = parseInt(Math.floor(TINT));
  TSEG = 2.0 * (TINT - NSEG) - 1.0;
  NumCoef = this.CC[startIdx + NSEG][4];
  
  var cT;
  var cTdot;

  X = 0;
  Y = 0; 
  Z = 0;
  
  VX = 0;
  VY = 0; 
  VZ = 0;
  for(var i = 0; i < NumCoef; i++)
  {
   cT = Math.Tn(i, TSEG);
   X += (this.CC[startIdx + NSEG][5 + i] * cT);
   Y += (this.CC[startIdx + NSEG][26 + i] * cT);
   Z += (this.CC[startIdx + NSEG][47 + i] * cT);

   if(calcVelocity == true)
   {
    cTdot = Math.TnDot(i, TSEG);
    VX += (this.CC[startIdx + NSEG][5 + i] * cTdot);
    VY += (this.CC[startIdx + NSEG][26 + i] * cTdot);
    VZ += (this.CC[startIdx + NSEG][47 + i] * cTdot);
    }
  }
  
  this.SV.X = Vector.FromKartezian(X * 1000.0 / Konst.AU, Y * 1000.0 / Konst.AU, Z * 1000.0 / Konst.AU);
  
  if(calcVelocity == true)
  {
   this.SV.V = Vector.FromKartezian(VX, VY, VZ);
   this.SV.V = Vector.MULT(2 * 1000 * SubIntervals / (D * 24 * 3600), this.SV.V);
  } 
 }
 
 return isCalc;
}
//========================================================
Body.prototype.ToHAE = function()
{
 this.SV = StateVector.MMULT(Coords.ICRFtoHAE, this.SV); 
}
//========================================================
Body.prototype.ToICRF = function()
{
 this.SV = StateVector.MMULT(Coords.HAEtoICRF, this.SV); 
}
//========================================================
Body.prototype.OPCStoICRF = function()
{
 this.SV = StateVector.MMULT(this.OE.OPCStoHAE, this.SV); 
 this.SV = StateVector.MMULT(Coords.HAEtoICRF, this.SV); 
}
//========================================================
Body.Elongation = function(target, observer, sun)
{
 var Elon = 0;
 var vSun = Vector.DIFF(sun, observer);
 var vBody = Vector.DIFF(target, observer);
 var v = Vector.PROD(vSun, vBody);
 var i = 90 - v.Theta;
 var W = v.Fi + 90;
 var rotM = Matrix.MULT(Matrix.RotX(i), Matrix.RotZ(W));
 
 vSun  = Matrix.VMULT(rotM, vSun);
 vBody = Matrix.VMULT(rotM, vBody);
 Elon = (vBody.Fi - vSun.Fi) * Math.Sign(90 - i);
 
 return Angle.ToAngle180(Elon);
}
//========================================================
Body.Phase = function(target, observer, sun)
{
 var ph = 0;
 var vBody = Vector.DIFF(observer, target);
 var vSun = Vector.DIFF(sun, target);
 var v = Vector.PROD(vSun , vBody);
 var i = 90 - v.Theta;
 var W = v.Fi + 90;
 var rotM = Matrix.MULT(Matrix.RotX(i), Matrix.RotZ(W));
 vBody = Matrix.VMULT(rotM, vBody);
 vSun = Matrix.VMULT(rotM, vSun);
 ph = (vBody.Fi - vSun.Fi) * Math.Sign(90 - i);
    
 return Angle.ToAngle180(ph);
}
//========================================================
Body.Magnitude = function(H, phase, r, rtsun)
{
 var m;
 
 var f = Math.abs(phase);
 var p = 2 * ((1 - f / 180) * Math.cos(f * Math.PI / 180) + Math.sin(f * Math.PI / 180) / Math.PI) / 3;
 var P1 = Math.pow(Math.cos(f * Math.PI / 360), 2);
 var m1 = H + 2.5 * Math.Log10(r * r * rtsun * rtsun / p);
 var m2 = H + 5 * Math.Log10(r) + 5 * Math.Log10(rtsun) - 2.5 * Math.Log10(P1);

 m = (m1 + m2) / 2;
 
 return m1;
}
//========================================================
Body.Illum = function(phase)
{
 return Math.pow(Math.cos(Angle.ToRad(phase) / 2), 2) * 100;
}
//========================================================

//******************************************************** 
// Inicializace dat 
//********************************************************
var EarthBody = new Body(1.000002610000000E+00,	1.538672142368240E-10,	1.671123000000000E-02,	-1.202464065708420E-09,	-1.531000000000000E-05,	-3.544607802874740E-07,	0.000000000000000E+00,	 0.000000000000000E+00,	 1.029376819300000E+02,	8.850749897330590E-06,	 1.004645716600000E+02,	9.856091019797400E-01, 0, 90);
EarthBody.CC = earthCC;

var MarsBody  = new Body(1.523710340000000E+00,	5.056810403832990E-10,	9.339410000000000E-02,	 2.157973990417520E-09,	 1.849691420000000E+00,	-2.226231348391510E-07,	4.955953891000000E+01,	-8.010223956194390E-06,	-2.394362959000000E+01,	1.216730677618070E-05,	-4.553432050000000E+00,	5.240329277204660E-01, 317.67134, 52.88071);
MarsBody.CC = marsCC;

var SunBody = new Body(0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0);
SunBody.CC = sunCC;

var MoonBody = new Body(0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0);
MoonBody.CC = moonCC;
//========================================================

