﻿//******************************************************** 
// Rozšíření třídy Math
//******************************************************** 
Math.Sign = function(x)
{
 return x / Math.abs(x);
}
//========================================================
Math.Trunc = function(x)
{
 return parseInt(x);
}
//========================================================
Math.Frac = function(x)
{
 return x - parseInt(x);
}
//========================================================
Math.Log10 = function(x)
{
 return Math.log(x) / Math.LN10;
}
//========================================================
Math.cosh = function(x)
{
 return (Math.exp(x) + Math.exp(-x)) / 2.0;
}
//========================================================
Math.Tn = function(n, x)
{
 if(n == 0)
  return 1.0;
 if(n == 1)
  return x;

 return 2.0 * x * Math.Tn(n - 1, x) - Math.Tn(n - 2, x);
}
//========================================================
Math.TnDot = function(n, x)
{
 if(n == 0)
  return 0.0;
 if(n == 1)
  return 1.0;

 return 2.0 * x * Math.TnDot(n - 1, x) + 2.0 * Math.Tn(n - 1, x) - Math.TnDot(n - 2, x);
}
//========================================================
Math.MOD = function(x1, x2)
{
 var mod = x1 - (Math.floor(x1 / x2) * x2);
 if(mod >= 0)
 {
  return mod;
 }
 else
 {
  return mod + x2;
 }
}
//========================================================
Math.FMODPI2 = function(x)
{
 return Math.MOD(x, (Math.PI * 2.0));
}
//========================================================

//******************************************************** 
// Pomocné úhlové funkce
//******************************************************** 
function Angle()
{
}
//========================================================
Angle.TORAD = Math.PI / 180.0;
Angle.TODEG = 180.0 / Math.PI;
//========================================================
Angle.ToRad = function(deg)
{
 return deg * Angle.TORAD;
}
//========================================================
Angle.ToDeg = function(rad)
{
 return rad * Angle.TODEG;
}
//========================================================
Angle.ToAngle180 = function(deg)
{
 deg = Angle.ToAngle360(deg);

 if(deg > 180)
 {
  deg -= 360;
 }
   
 return deg;
}
//========================================================
Angle.ToAngle360 = function(deg)
{
 if(deg >= 360)
  deg -= parseInt(deg / 360.0) * 360;
 
 if(deg < 0)
  deg += parseInt(-deg / 360 + 1.0) * 360;
 
 return deg;
}
//========================================================
Angle.ToHours24 = function(hour)
{
 if(hour >= 24)
  hour -= parseInt(hour / 24.0) * 24;
 else
  hour += parseInt(-hour / 24.0 + 1) * 24;
 return hour;
}
//========================================================
Angle.GetAngleRad = function(sinx, cosx)
{
 if(Math.abs(cosx) == 1)
    sinx = 0;

 var angle = Math.asin(sinx);

 if(sinx >= 0 && cosx < 0) angle = Math.PI - angle;
 else if(sinx < 0 && cosx < 0) angle = Math.abs(angle) + Math.PI;
 else if(sinx < 0 && cosx > 0) angle += 2.0 * Math.PI;

 if(angle == 2.0 * Math.PI) angle = 0;

 return angle;
}
//============================================================
Angle.GetAngle = function(sinx, cosx)
{
 return Angle.ToDeg(Angle.GetAngleRad(sinx, cosx));
}
//============================================================
Angle.Format = function(angle, isSec)
{
 var sign = Math.Sign(angle);
 angle = Math.abs(angle);
 var deg = Math.Trunc(angle);
 var amin = Math.Trunc((angle - deg) * 60);
 var asec = Math.Trunc((angle - deg) * 3600 - amin * 60); 
 deg = ((sign < 0) ? "-" : "") + deg + "°" + ((amin < 10) ? ("0" + amin) : amin) + "'";
 
 if(isSec == 1)
   deg = deg + ((asec < 10) ? ("0" + asec) : asec) + "\"";
 
 return deg;
}
//============================================================
Angle.FormatHMS = function(angle, htmlclass)
{
 angle = Angle.ToAngle360(angle) / 15;
 var deg = Math.Trunc(angle);
 var amin = Math.Trunc((angle - deg) * 60);
 var asec = Math.Trunc((angle - deg) * 3600 - amin * 60); 
 if(htmlclass == null)
 {
  deg = deg + "h" + ((amin < 10) ? ("0" + amin) : amin) + "m";
  deg = deg + ((asec < 10) ? ("0" + asec) : asec) + "s";
 }
 else
 {
  var spans = "<span class='" + htmlclass + "'>";
  var spane = "</span>";
  deg = ((deg < 10) ? ("0" + deg) : deg) + spans + "h" + spane + ((amin < 10) ? ("0" + amin) : amin) + spans + "m" + spane;
  deg = deg + ((asec < 10) ? ("0" + asec) : asec) + spans + "s" + spane;
 }
 
 return deg;
}
//============================================================

//******************************************************** 
// Vektory 3D 
//********************************************************
function Vector()
{
 this.X = new Array(3);
 
 for(var i = 0; i < 3; i++) this.X[i] = 0.0;
  
 this.R = 0.0;
 this.Fi = 0.0;
 this.Theta = 0.0;
}
//============================================================
Vector.FromSpherical = function(r, fi, theta)
{
 var v = new Vector();
 
 v.R = r; 
 v.Fi = fi; 
 v.Theta = theta; 
 
 v.ToKartezian();
 
 return v;
}
//============================================================
Vector.FromKartezian = function(x, y, z)
{
 var v = new Vector();
 
 v.X[0] = x; 
 v.X[1] = y; 
 v.X[2] = z; 
 
 v.ToSpherical();
 
 return v;
}
//============================================================
Vector.prototype.FromThis = function()
{
 var v = new Vector();
 
 v.X[0] = this.X[0]; 
 v.X[1] = this.X[1]; 
 v.X[2] = this.X[2]; 
 v.ToSpherical();
 
 return v;
}
//============================================================
Vector.prototype.Size = function()
{
 var r = 0.0;

 for(var i = 0; i < 3; i++) r += Math.pow(this.X[i], 2);

 return Math.sqrt(r);
}
//============================================================
Vector.prototype.SizeXY = function()
{
 var r = 0.0;

 for(var i = 0; i < 2; i++) r += Math.pow(this.X[i], 2);

 return Math.sqrt(r);
}
//============================================================  
Vector.prototype.ToKartezian = function()
{
 var Rxy = this.R * Math.cos(Angle.ToRad(this.Theta));
 
 this.X[2] = this.R * Math.sin(Angle.ToRad(this.Theta));
 this.X[1] = Rxy * Math.sin(Angle.ToRad(this.Fi));
 this.X[0] = Rxy * Math.cos(Angle.ToRad(this.Fi));
}
//============================================================
Vector.prototype.ToSpherical = function()
{
 var Rxy = this.SizeXY();
 this.R = this.Size();

 if(Math.abs(this.R) > 0)
 {
  this.Theta = Angle.ToDeg(Math.asin(this.X[2] / this.R));
  this.Fi = Angle.GetAngle(this.X[1] / Rxy, this.X[0] / Rxy);
 }
 else
 {
  this.Theta = 0;
  this.Fi = 0;
 }
}
//============================================================
// Součet
Vector.SUM = function(a, b) 
{
 var v = new Vector();
 
 for(var i = 0; i < 3; i++) v.X[i] = a.X[i] + b.X[i];
 v.ToSpherical();
 
 return v;
}
//========================================================
// Rozdíl
Vector.DIFF = function(a, b) 
{
 var v = new Vector();
 
 for(var i = 0; i < 3; i++) v.X[i] = a.X[i] - b.X[i];
 v.ToSpherical();
 
 return v;
}
//========================================================
// Vektorový součin
Vector.PROD = function(a, b) 
{
 var c = new Vector();

 c.X[0] = a.X[1] * b.X[2] - a.X[2] * b.X[1];
 c.X[1] = a.X[2] * b.X[0] - a.X[0] * b.X[2];
 c.X[2] = a.X[0] * b.X[1] - a.X[1] * b.X[0];
 c.ToSpherical();

 return c;
}
//========================================================
// Skalární součin
Vector.DOTPROD = function(a, b) 
{
 var c = 0;

 c = a.X[0] * b.X[0] + a.X[1] * b.X[1] + a.X[2] * b.X[2];

 return c;
}
//========================================================
Vector.MULT = function(n, v) 
{
 var c = new Vector();

 c.X[0] = n * v.X[0];
 c.X[1] = n * v.X[1];
 c.X[2] = n * v.X[2];
 c.ToSpherical();

 return c;
}
//========================================================
Vector.MMULT = function(M, v) 
{
 return Matrix.VMULT(M, v);
}
//========================================================
Vector.Unitary = function(v) 
{
 var c = v.FromThis();
 c = Vector.MULT(1.0 / v.R, c);
 c.ToSpherical();

 return c;
}
//========================================================

//********************************************************
// Stavový vektor 
//********************************************************
function StateVector()
{
 this.X = new Vector();
 this.V = new Vector();
}
//========================================================
StateVector.prototype.FromThis = function()
{
 var c = new StateVector();
 
 c.X = this.X.FromThis();
 c.V = this.V.FromThis();
 
 return c;
}
//========================================================
StateVector.SUM = function(a, b)
{
 var c = new StateVector();
 
 c.X = Vector.SUM(a.X, b.X);
 c.V = Vector.SUM(a.V, b.V);
 
 return c;
}
//========================================================
StateVector.DIFF = function(a, b)
{
 var c = new StateVector();
 
 c.X = Vector.DIFF(a.X, b.X);
 c.V = Vector.DIFF(a.V, b.V);
 
 return c;
}
//========================================================
StateVector.MMULT = function(M, a)
{
 var c = new StateVector();
 
 c.X = Matrix.VMULT(M, a.X);
 c.V = Matrix.VMULT(M, a.V);
 
 return c;
}
//========================================================
StateVector.MULT = function(n, a)
{
 var c = new StateVector();
 
 c.X = Vector.MULT(n, a.X);
 c.V = Vector.MULT(n, a.V);
 
 return c;
}
//========================================================

//******************************************************** 
// Matice 3D 
//********************************************************
function Matrix()
{
 this.M = new Array(3);
 
 for(var i = 0; i < 3; i++) this.M[i] = new Array(3);
 
 for(var i = 0; i < 3; i++)
  for(var j = 0; j < 3; j++)
   this.M[i][j] = 0.0;
}
//========================================================
Matrix.prototype.SetToIdentity = function()
{
 for(var i = 0; i < 3; i++)
  for(var j = 0; j < 3; j++) this.M[i][j] = (i == j) ? 1.0 : 0.0;
}
//============================================================
Matrix.Identity = function() 
{
 var M = new Matrix();
 
 for(var i = 0; i < 3; i++)
  for(var j = 0; j < 3; j++) M.M[i][j] = (i == j) ? 1.0 : 0.0;
  
 return M; 
}
//============================================================
Matrix.MULT = function(a, b) 
{
 var c = new Matrix();
 
 for(var i = 0; i < 3; i++)
  for(var j = 0; j < 3; j++)
   for(var k = 0; k < 3; k++) c.M[i][j] += a.M[i][k] * b.M[k][j];
 
 return c;
}
//========================================================
Matrix.VMULT = function(M, v) 
{
 var c = new Vector();

 for(var i = 0; i < 3; i++)
  for(var k = 0; k < 3; k++) c.X[i] += M.M[i][k] * v.X[k];

 c.ToSpherical();

 return c;
}
//========================================================
Matrix.RotX = function(fi)
{
 var c = new Matrix();
 fi = Angle.ToRad(fi);

 c.SetToIdentity();
 c.M[1][1] = Math.cos(fi);
 c.M[1][2] = Math.sin(fi);
 c.M[2][1] = -Math.sin(fi);
 c.M[2][2] = Math.cos(fi);

 return c;
}
//============================================================
Matrix.RotZ = function(fi)
{
 var c = new Matrix();
 fi = Angle.ToRad(fi);

 c.SetToIdentity();
 c.M[0][0] = Math.cos(fi);
 c.M[0][1] = Math.sin(fi);
 c.M[1][0] = -Math.sin(fi);
 c.M[1][1] = Math.cos(fi);

 return c;
}
//============================================================
Matrix.RotY = function(fi)
{
 var c = new Matrix();
 fi = Angle.ToRad(fi);

 c.SetToIdentity();
 c.M[0][0] = Math.cos(fi);
 c.M[0][2] = Math.sin(fi);
 c.M[2][0] = -Math.sin(fi);
 c.M[2][2] = Math.cos(fi);

 return c;
}
//============================================================
Matrix.RotZdot = function(fi, w)
{
 var c = new Matrix();
 fi = Angle.ToRad(fi);
 w = Angle.ToRad(w);

 c.M[0][0] = -Math.sin(fi) * w;
 c.M[0][1] = Math.cos(fi) * w;
 c.M[1][0] = -Math.cos(fi) * w;
 c.M[1][1] = -Math.sin(fi) * w;

 return c;
}
//============================================================