﻿//******************************************************** 
// Systémy souřadnic 
//********************************************************
function Coords()
{
}
//========================================================
Coords.ToMercator = function(y)
{
 return Y = 180 * Math.log( (1 + Math.sin(Angle.ToRad(y))) / (1 - Math.sin(Angle.ToRad(y)))) / (2 * Math.PI);
}
//========================================================
Coords.FromMercator = function(y)
{
 return Angle.ToDeg(2 * Math.atan(Math.exp(Angle.ToRad(y))) - Math.PI / 2);
}
//========================================================
Coords.ICRFtoHAE = Matrix.RotX(Konst.EIN);
//========================================================
Coords.HAEtoICRF = Matrix.RotX(-Konst.EIN);
//========================================================
Coords.ITRFtoHCS = function(lon, lat)
{
 var M = Matrix.RotZ(lon)
 var u = Matrix.Identity();
 u.M[1][1] = -1;
 
 M = Matrix.MULT(Matrix.RotY(lat - 90), M);
 M = Matrix.MULT(Matrix.RotZ(180), M);
 M = Matrix.MULT(u, M);
 
 return M;
}
//========================================================
Coords.GCRFtoITRF = function(JDTDB, deltaUT)
{
 var GAST = EarthModel.GAST(JDTDB, deltaUT);
 M = Matrix.RotZ(GAST);
 return M;
}
//========================================================

//******************************************************** 
// Model WGS84 
//********************************************************
function EarthModel()
{
}
//========================================================
EarthModel.a  = 6378137;           // rovníkový poloměr
EarthModel.b  = 6356752.3142;      // polární poloměr
EarthModel.e2 = 6.69437999014E-3;  // druhá mocnina excentricity
EarthModel.f = 1 / 298.257223563;  // zploštění
EarthModel.OBL = 84381.406;        // sklon rotační osy k ekliptice k J2000 (arcsec)
EarthModel.W = 360.98564736596;    // úhlová rychlost rotace °/d
//======================================================== 
EarthModel.VectorITRF = function(Lon, Lat, Alt)
{
 var XN;
 var X, Y, Z;
 var v;
 
 Lon = Angle.ToRad(Lon);
 Lat = Angle.ToRad(Lat);
 XN = Math.sqrt(1 - EarthModel.e2 * Math.pow(Math.sin(Lat), 2));

 X = (EarthModel.a / XN + Alt) * Math.cos(Lon) * Math.cos(Lat);
 Y = (EarthModel.a / XN + Alt) * Math.sin(Lon) * Math.cos(Lat);
 Z = (EarthModel.a * (1 - EarthModel.e2) / XN + Alt) * Math.sin(Lat)
 v = Vector.FromKartezian(X, Y, Z);
 
 return v;
}
//======================================================== 
EarthModel.VectorWGS = function(ECEF)
{
 var X, Y, Z;
 var a, f, b, e2, ep2, r2, r;
 var E2, F, G, c, s, P, Q, ro;
 var tmp, U, V, zo;
 var h, phi;
 var retV;
 
 X = ECEF.X[0];
 Y = ECEF.X[1];
 Z = ECEF.X[2];
 a = EarthModel.a;
 f = EarthModel.f;
 b = EarthModel.b;
 e2 = EarthModel.e2;
 ep2 = f * (2 - f) / ((1 - f) * (1 - f));
 r2 = X * X + Y * Y;
 r = Math.sqrt(r2);
 E2 = a * a - b * b;
 F = 54 * b * b * Z * Z;
 G = r2 + (1 - e2) * Z * Z - e2 * E2;
 c = (e2 * e2 * F * r2) / (G * G * G);
 s = Math.pow(1 + c + Math.sqrt(c * c + 2 * c), 1.0 / 3);
 P = F / (3 * Math.pow(s + 1.0 / s + 1, 2) * G * G);
 Q = Math.sqrt(1 + 2 * e2 * e2 * P);
 ro = -(e2 * P * r) / (1 + Q) + Math.sqrt((a * a / 2) * (1 + 1.0 / Q) - ((1 - e2) * P * Z * Z) /(Q * (1 + Q)) - P * r2 / 2);
 tmp = Math.pow(r - e2 * ro, 2);
 U = Math.sqrt(tmp + Z * Z);
 V = Math.sqrt(tmp + (1 - e2) * Z * Z);
 zo = (b * b * Z) / (a * V);
 
 h = U * (1 - b * b / (a * V));
 phi = Angle.ToDeg(Math.atan((Z + ep2 * zo) / r));
 
 retV = Vector.FromSpherical(h, ECEF.Fi, phi);
 
 return retV;
}
//========================================================
EarthModel.GMST = function(JDTDB, deltaUT)
{
 var JDUT = JDTDB - deltaUT / 86400;
 var DUT = JDUT - 2451545.0;
 var T = (JDTDB - 2451545.0) / 36525;
 var T2 = T * T;
 var T3 = T2 * T;
 var T4 = T3 * T;
 var T5 = T4 * T;
 var ERA = 0.7790572732640 + 0.00273781191135448 * DUT + Math.Frac(JDUT);
 var GMST = (86400 * ERA * 15 + 0.014506 + 4612.156534 * T + 1.3915817 * T2 - 0.00000044 * T3 - 0.000029956 * T4 - 0.0000000368 * T5) / 3600;
 
 return GMST;
}
//========================================================
EarthModel.GAST = function(JDTDB, deltaUT)
{
 var JDUT = JDTDB - deltaUT;
 var T = (JDUT - 2451545.0) / 36525.0;
 var L = 280.4665 + 36000.7698 * T;
 var L1 = 218.3165 + 481267.8813 * T;
 var W = 125.04452 - 1934.136261 * T;
 
 L = Angle.TORAD * L;
 L1 = Angle.TORAD * L1;
 W = Angle.TORAD * W;
 
 var Eps = EarthModel.MeanObliquity(JDTDB) / 3600;
 var dPsi = -17.20 * Math.sin(W) - 1.32 * Math.sin(2 * L) - 0.23 * Math.sin(2 * L1) + 0.21 * Math.sin(2 * W);
 var dEps = 9.20 * Math.cos(W) + 0.57 * Math.cos(2 * L) + 0.10 * Math.cos(2 * L1) - 0.09 * Math.cos(2 * W);
 
 dPsi = dPsi / 3600;
 dEps = (dEps + EarthModel.MeanObliquity(JDTDB)) / 3600;
 
 var GAST = EarthModel.GMST(JDTDB, deltaUT);
 GAST += dPsi * Math.cos(dEps * Angle.TORAD);
 
 return GAST;
}
//========================================================
// v arcsec !
EarthModel.MeanObliquity = function(JDTDB)
{
 var T = (JDTDB - 2451545.0) / 36525;
 var T2 = T * T;
 var T3 = T2 * T;
 var T4 = T3 * T;
 var T5 = T4 * T;
 var e = EarthModel.OBL - 46.836769 * T - 0.0001831 * T2 + 0.00200340 * T3 - 0.000000576 * T4 - 0.0000000434 * T5;
 
 return e;
}
//========================================================
// Precesní matice
EarthModel.PMX = function(JDTDB)
{
 var T = (JDTDB - 2451545.0) / 36525.0;
 var T2 = T * T;
 var T3 = T2 * T;
 var T4 = T3 * T;
 var T5 = T4 * T;
 var M;
 
 var psi = (5038.481507 * T - 1.0790069 * T2 - 0.00114045 * T3 + 0.000132851 * T4 - 0.0000000951 * T5) / 3600;
 var om  = (84381.406 - 0.025754 * T + 0.0512623 * T2 - 0.00772503 * T3 - 0.000000467 * T4 + 0.0000003337 * T5) / 3600;
 var chi = (10.556403 * T - 2.3814292 * T2 - 0.00121197 * T3 + 0.000170663 * T4 - 0.0000000560 * T5) / 3600;
 var e0 = EarthModel.OBL / 3600;
 
 M = Matrix.RotX(e0);
 M = Matrix.MULT(Matrix.RotZ(-psi), M);
 M = Matrix.MULT(Matrix.RotX(-om), M);
 M = Matrix.MULT(Matrix.RotZ(chi), M);
 
 return M;
}
//========================================================
// Frame-Bias matice
EarthModel.BMX = function()
{
 var da0 = -22.9;
 var ksi0 = 9.1;
 var eta0 = -19.9;
 var koef = 3600*1000; // převod mas na stupně
 var M;
 
 da0 /= koef;
 ksi0 /= koef;
 eta0 /= koef;
 
 M = Matrix.RotZ(da0);
 M = Matrix.MULT(Matrix.RotY(ksi0), M);
 M = Matrix.MULT(Matrix.RotX(-eta0), M);
 
 return M;
}
//========================================================
// Nutační matice
EarthModel.NMX = function(JDTDB, deltaUT)
{
 var M;
 var JDUT = JDTDB - deltaUT;
 var T = (JDUT - 2451545.0) / 36525.0;
 var L = 280.4665 + 36000.7698 * T;
 var L1 = 218.3165 + 481267.8813 * T;
 var W = 125.04452 - 1934.136261 * T;
 
 L = Angle.TORAD * L;
 L1 = Angle.TORAD * L1;
 W = Angle.TORAD * W;
 
 var Eps = EarthModel.MeanObliquity(JDTDB) / 3600;
 var dPsi = -17.20 * Math.sin(W) - 1.32 * Math.sin(2 * L) - 0.23 * Math.sin(2 * L1) + 0.21 * Math.sin(2 * W);
 var dEps = 9.20 * Math.cos(W) + 0.57 * Math.cos(2 * L) + 0.10 * Math.cos(2 * L1) - 0.09 * Math.cos(2 * W);
 
 dPsi = dPsi / 3600;
 dEps = dEps / 3600;
 
 M = Matrix.RotX(Eps);
 M = Matrix.MULT(Matrix.RotZ(-dPsi), M);
 M = Matrix.MULT(Matrix.RotX(-(Eps + dEps)), M);
 
 return M;
}
//========================================================

//******************************************************** 
// Model MOLA 
//********************************************************
function MarsModel()
{
}
//========================================================
MarsModel.R  = 3389508;              // střední poloměr
MarsModel.a  = 3396190;              // rovníkový poloměr
MarsModel.b  = 3376200;              // polární poloměr
MarsModel.e2 = 1.17373700261073E-02; // druhá mocnina excentricity
MarsModel.f = 1 / 169.894447223611;  // zploštění
MarsModel.OBL = 90681.9012;          // sklon rotační osy k ekliptice k J2000 (arcsec)
MarsModel.W = 350.8919852;          // úhlová rychlost rotace °/d
MarsModel.W0 = 176.630;              // Meridian J2000
MarsModel.RA0 = 317.68143;           // Pól J2000
MarsModel.DE0 = 52.88650;
MarsModel.RAdot = -0.1061;           // pohyb pólů °/JC
MarsModel.DEdot = -0.0609;
//======================================================== 
// Střední hvězdný čas Airy-0
MarsModel.MST = function(delta2000)
{
 return 313.476 + MarsModel.W * delta2000;
}
//========================================================
MarsModel.VectorACRF = function(Lon, Lat, Alt)
{
 var XN;
 var X, Y, Z;
 var v;
 
 Alt *= 1000;
 Lon = Angle.ToRad(Lon);
 Lat = Angle.ToRad(Lat);
 XN = Math.sqrt(1 - MarsModel.e2 * Math.pow(Math.sin(Lat), 2));

 X = (MarsModel.a / XN + Alt) * Math.cos(Lon) * Math.cos(Lat);
 Y = (MarsModel.a / XN + Alt) * Math.sin(Lon) * Math.cos(Lat);
 Z = (MarsModel.a * (1 - MarsModel.e2) / XN + Alt) * Math.sin(Lat)
 v = Vector.FromKartezian(X, Y, Z);
 
 return v;
}
//======================================================== 
// Rotační matice pro převod planetogr. souřadnic do ICRF
MarsModel.ACRFtoICRF = function(W, RA, DE)
{
 var M = Matrix.RotZ(-W);
 
 M = Matrix.MULT(Matrix.RotX(-(90 - DE)), M);
 M = Matrix.MULT(Matrix.RotZ(-(90 + RA)), M);
 
 return M;
}
//========================================================
// Rotační matice pro převod ICRF do planetogr. souřadnic
MarsModel.ICRFtoACRF = function(W, RA, DE)
{
 var M = Matrix.RotZ(90 + RA);
 
 M = Matrix.MULT(Matrix.RotX(90 - DE), M);
 M = Matrix.MULT(Matrix.RotZ(W), M);
 
 return M;
}
//========================================================
// Poloha v ICRF (počátek ve středu Marsu)
MarsModel.VectorICRF = function(delta, lon, lat, alt)
{
 var W = 176.630 + 350.89198226 * delta;
 var RA = 317.68143 - 0.1061 * delta / 36525;
 var DE = 52.88650 - 0.0609 * delta / 36525;
 var v = Vector.MULT(1.0 / Konst.AU, MarsModel.VectorACRF(lon, lat, alt));
 
 v = Matrix.VMULT(MarsModel.ACRFtoICRF(W, RA, DE), v);
 return v;
}
//========================================================
// Rotační matice z ICRF do místních horizont. souř.
MarsModel.ICRFtoHCS = function(delta, lon, lat, alt)
{
 var W = 176.630 + 350.89198226 * delta;
 var RA = 317.68143 - 0.1061 * delta / 36525;
 var DE = 52.88650 - 0.0609 * delta / 36525;
 var M = MarsModel.ICRFtoACRF(W, RA, DE);

 M = Matrix.MULT(Coords.ITRFtoHCS(lon, lat), M);
 return M;
}
//========================================================